import sys
from PyQt6.QtWidgets import QApplication, QStyle, QStyleFactory
import fmtransfer
# import importlib.metadata
# import requests

def _main() -> None:
    app = QApplication(sys.argv)
    style: QStyle = app.style()
    if style.name() == "windows11":
        app.setStyle(QStyleFactory.create('Fusion'))
    fm_transfer = fmtransfer.FmTransfer()
    fm_transfer.show()
    sys.exit(app.exec())

def _check_versions() -> None:
    # url = "https://pypi.org/pypi/gg-transfer/json"
    # res = requests.get(url)
    # json_data = res.json()
    # print("gg-transfer", json_data["info"]["version"], importlib.metadata.version("gg-transfer"))
    # url = "https://pypi.org/pypi/quiet-transfer/json"
    # res = requests.get(url)
    # json_data = res.json()
    # print("quiet-transfer", json_data["info"]["version"], importlib.metadata.version("quiet-transfer"))
    # url = "https://pypi.org/pypi/ggwave-wheels/json"
    # res = requests.get(url)
    # json_data = res.json()
    # print("ggwave-wheels", json_data["info"]["version"], importlib.metadata.version("ggwave-wheels"))
    # url = "https://pypi.org/pypi/fm-transfer/json"
    # res = requests.get(url)
    # json_data = res.json()
    # print("fm_transfer", json_data["info"]["version"], fmtransfer.__version__)
    pass


if __name__ == "__main__":
    _main()



# Form implementation generated from reading ui file 'fm-transfer.ui'
#
# Created by: PyQt6 UI code generator 6.7.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_FmTransfer(object):
    def setupUi(self, FmTransfer):
        FmTransfer.setObjectName("FmTransfer")
        FmTransfer.setWindowModality(QtCore.Qt.WindowModality.NonModal)
        FmTransfer.resize(573, 731)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(FmTransfer.sizePolicy().hasHeightForWidth())
        FmTransfer.setSizePolicy(sizePolicy)
        FmTransfer.setMinimumSize(QtCore.QSize(530, 630))
        FmTransfer.setMaximumSize(QtCore.QSize(800, 800))
        icon = QtGui.QIcon.fromTheme("QIcon::ThemeIcon::MediaSeekForward")
        FmTransfer.setWindowIcon(icon)
        FmTransfer.setAutoFillBackground(False)
        FmTransfer.setDocumentMode(False)
        self.centralwidget = QtWidgets.QWidget(parent=FmTransfer)
        self.centralwidget.setObjectName("centralwidget")
        self.centralGridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.centralGridLayout.setObjectName("centralGridLayout")
        self.qhBoxHorizontalLayout = QtWidgets.QHBoxLayout()
        self.qhBoxHorizontalLayout.setSizeConstraint(QtWidgets.QLayout.SizeConstraint.SetDefaultConstraint)
        self.qhBoxHorizontalLayout.setObjectName("qhBoxHorizontalLayout")
        self.pttGroupBox = QtWidgets.QGroupBox(parent=self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pttGroupBox.sizePolicy().hasHeightForWidth())
        self.pttGroupBox.setSizePolicy(sizePolicy)
        self.pttGroupBox.setMinimumSize(QtCore.QSize(0, 0))
        self.pttGroupBox.setMaximumSize(QtCore.QSize(130, 100))
        self.pttGroupBox.setObjectName("pttGroupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.pttGroupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.pttReleasedRadioButton = QtWidgets.QRadioButton(parent=self.pttGroupBox)
        self.pttReleasedRadioButton.setChecked(True)
        self.pttReleasedRadioButton.setAutoExclusive(True)
        self.pttReleasedRadioButton.setObjectName("pttReleasedRadioButton")
        self.verticalLayout_3.addWidget(self.pttReleasedRadioButton)
        self.pttPressedRadioButton = QtWidgets.QRadioButton(parent=self.pttGroupBox)
        self.pttPressedRadioButton.setAutoExclusive(True)
        self.pttPressedRadioButton.setObjectName("pttPressedRadioButton")
        self.verticalLayout_3.addWidget(self.pttPressedRadioButton)
        self.signalLogic = QtWidgets.QCheckBox(parent=self.pttGroupBox)
        self.signalLogic.setObjectName("signalLogic")
        self.verticalLayout_3.addWidget(self.signalLogic)
        self.qhBoxHorizontalLayout.addWidget(self.pttGroupBox)
        self.toolGroupBox = QtWidgets.QGroupBox(parent=self.centralwidget)
        self.toolGroupBox.setMaximumSize(QtCore.QSize(130, 100))
        self.toolGroupBox.setObjectName("toolGroupBox")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.toolGroupBox)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.ggRadioButton = QtWidgets.QRadioButton(parent=self.toolGroupBox)
        self.ggRadioButton.setChecked(True)
        self.ggRadioButton.setObjectName("ggRadioButton")
        self.verticalLayout_5.addWidget(self.ggRadioButton)
        self.quietRadioButton = QtWidgets.QRadioButton(parent=self.toolGroupBox)
        self.quietRadioButton.setObjectName("quietRadioButton")
        self.verticalLayout_5.addWidget(self.quietRadioButton)
        self.qhBoxHorizontalLayout.addWidget(self.toolGroupBox)
        self.protocolGroupBox = QtWidgets.QGroupBox(parent=self.centralwidget)
        self.protocolGroupBox.setMaximumSize(QtCore.QSize(180, 100))
        self.protocolGroupBox.setObjectName("protocolGroupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.protocolGroupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.ggProtocolComboBox = QtWidgets.QComboBox(parent=self.protocolGroupBox)
        self.ggProtocolComboBox.setObjectName("ggProtocolComboBox")
        self.ggProtocolComboBox.addItem("")
        self.ggProtocolComboBox.addItem("")
        self.ggProtocolComboBox.addItem("")
        self.ggProtocolComboBox.addItem("")
        self.ggProtocolComboBox.addItem("")
        self.ggProtocolComboBox.addItem("")
        self.ggProtocolComboBox.addItem("")
        self.ggProtocolComboBox.addItem("")
        self.ggProtocolComboBox.addItem("")
        self.verticalLayout.addWidget(self.ggProtocolComboBox)
        self.quietProtocolComboBox = QtWidgets.QComboBox(parent=self.protocolGroupBox)
        self.quietProtocolComboBox.setObjectName("quietProtocolComboBox")
        self.quietProtocolComboBox.addItem("")
        self.quietProtocolComboBox.addItem("")
        self.quietProtocolComboBox.addItem("")
        self.quietProtocolComboBox.addItem("")
        self.quietProtocolComboBox.addItem("")
        self.quietProtocolComboBox.addItem("")
        self.quietProtocolComboBox.addItem("")
        self.verticalLayout.addWidget(self.quietProtocolComboBox)
        self.qhBoxHorizontalLayout.addWidget(self.protocolGroupBox)
        self.signalGroupBox = QtWidgets.QGroupBox(parent=self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.signalGroupBox.sizePolicy().hasHeightForWidth())
        self.signalGroupBox.setSizePolicy(sizePolicy)
        self.signalGroupBox.setMinimumSize(QtCore.QSize(130, 0))
        self.signalGroupBox.setMaximumSize(QtCore.QSize(130, 100))
        self.signalGroupBox.setObjectName("signalGroupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.signalGroupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.dsrRadioButton = QtWidgets.QRadioButton(parent=self.signalGroupBox)
        self.dsrRadioButton.setChecked(True)
        self.dsrRadioButton.setAutoExclusive(True)
        self.dsrRadioButton.setObjectName("dsrRadioButton")
        self.verticalLayout_2.addWidget(self.dsrRadioButton)
        self.rtsRadioButton = QtWidgets.QRadioButton(parent=self.signalGroupBox)
        self.rtsRadioButton.setAutoExclusive(True)
        self.rtsRadioButton.setObjectName("rtsRadioButton")
        self.verticalLayout_2.addWidget(self.rtsRadioButton)
        self.qhBoxHorizontalLayout.addWidget(self.signalGroupBox)
        self.centralGridLayout.addLayout(self.qhBoxHorizontalLayout, 0, 0, 1, 1)
        self.messages = QtWidgets.QPlainTextEdit(parent=self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.messages.sizePolicy().hasHeightForWidth())
        self.messages.setSizePolicy(sizePolicy)
        self.messages.setAcceptDrops(False)
        self.messages.setUndoRedoEnabled(False)
        self.messages.setLineWrapMode(QtWidgets.QPlainTextEdit.LineWrapMode.NoWrap)
        self.messages.setReadOnly(True)
        self.messages.setCenterOnScroll(False)
        self.messages.setObjectName("messages")
        self.centralGridLayout.addWidget(self.messages, 1, 0, 1, 1)
        self.signalLedHorizontalLayout = QtWidgets.QHBoxLayout()
        self.signalLedHorizontalLayout.setObjectName("signalLedHorizontalLayout")
        self.checkSignalButton = QtWidgets.QPushButton(parent=self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Maximum, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkSignalButton.sizePolicy().hasHeightForWidth())
        self.checkSignalButton.setSizePolicy(sizePolicy)
        self.checkSignalButton.setMinimumSize(QtCore.QSize(98, 0))
        self.checkSignalButton.setObjectName("checkSignalButton")
        self.signalLedHorizontalLayout.addWidget(self.checkSignalButton)
        self.led = Led(parent=self.centralwidget)
        self.led.setOn(False)
        self.led.setLightColor(QtGui.QColor(255, 0, 0))
        self.led.setObjectName("led")
        self.signalLedHorizontalLayout.addWidget(self.led)
        self.serialComboBox = QtWidgets.QComboBox(parent=self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.serialComboBox.sizePolicy().hasHeightForWidth())
        self.serialComboBox.setSizePolicy(sizePolicy)
        self.serialComboBox.setObjectName("serialComboBox")
        self.signalLedHorizontalLayout.addWidget(self.serialComboBox)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.signalLedHorizontalLayout.addItem(spacerItem)
        self.pushButton = QtWidgets.QPushButton(parent=self.centralwidget)
        self.pushButton.setObjectName("pushButton")
        self.signalLedHorizontalLayout.addWidget(self.pushButton)
        self.centralGridLayout.addLayout(self.signalLedHorizontalLayout, 2, 0, 1, 1)
        self.sendFilehorizontalLayout = QtWidgets.QHBoxLayout()
        self.sendFilehorizontalLayout.setObjectName("sendFilehorizontalLayout")
        self.chooseSendFileButton = QtWidgets.QPushButton(parent=self.centralwidget)
        self.chooseSendFileButton.setMinimumSize(QtCore.QSize(98, 0))
        self.chooseSendFileButton.setObjectName("chooseSendFileButton")
        self.sendFilehorizontalLayout.addWidget(self.chooseSendFileButton)
        self.sendFileLineEdit = QtWidgets.QLineEdit(parent=self.centralwidget)
        self.sendFileLineEdit.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.sendFileLineEdit.setAcceptDrops(False)
        self.sendFileLineEdit.setReadOnly(True)
        self.sendFileLineEdit.setObjectName("sendFileLineEdit")
        self.sendFilehorizontalLayout.addWidget(self.sendFileLineEdit)
        self.sendFileButton = QtWidgets.QPushButton(parent=self.centralwidget)
        self.sendFileButton.setMinimumSize(QtCore.QSize(80, 0))
        self.sendFileButton.setObjectName("sendFileButton")
        self.sendFilehorizontalLayout.addWidget(self.sendFileButton)
        self.centralGridLayout.addLayout(self.sendFilehorizontalLayout, 3, 0, 1, 1)
        self.receiveHorizontalLayout = QtWidgets.QHBoxLayout()
        self.receiveHorizontalLayout.setObjectName("receiveHorizontalLayout")
        self.chooseRecvFileButton = QtWidgets.QPushButton(parent=self.centralwidget)
        self.chooseRecvFileButton.setMinimumSize(QtCore.QSize(98, 0))
        self.chooseRecvFileButton.setObjectName("chooseRecvFileButton")
        self.receiveHorizontalLayout.addWidget(self.chooseRecvFileButton)
        self.receiveFileLineEdit = QtWidgets.QLineEdit(parent=self.centralwidget)
        self.receiveFileLineEdit.setMinimumSize(QtCore.QSize(0, 0))
        self.receiveFileLineEdit.setAcceptDrops(False)
        self.receiveFileLineEdit.setReadOnly(True)
        self.receiveFileLineEdit.setObjectName("receiveFileLineEdit")
        self.receiveHorizontalLayout.addWidget(self.receiveFileLineEdit)
        self.receiveButton = QtWidgets.QPushButton(parent=self.centralwidget)
        self.receiveButton.setMinimumSize(QtCore.QSize(80, 0))
        self.receiveButton.setObjectName("receiveButton")
        self.receiveHorizontalLayout.addWidget(self.receiveButton)
        self.centralGridLayout.addLayout(self.receiveHorizontalLayout, 4, 0, 1, 1)
        self.progressBar = QtWidgets.QProgressBar(parent=self.centralwidget)
        self.progressBar.setObjectName("progressBar")
        self.centralGridLayout.addWidget(self.progressBar, 5, 0, 1, 1)
        self.msgHorizontalLayout = QtWidgets.QHBoxLayout()
        self.msgHorizontalLayout.setObjectName("msgHorizontalLayout")
        self.recvMsgButton = QtWidgets.QPushButton(parent=self.centralwidget)
        self.recvMsgButton.setObjectName("recvMsgButton")
        self.msgHorizontalLayout.addWidget(self.recvMsgButton)
        self.shortMsg = QtWidgets.QLineEdit(parent=self.centralwidget)
        self.shortMsg.setAcceptDrops(False)
        self.shortMsg.setMaxLength(140)
        self.shortMsg.setObjectName("shortMsg")
        self.msgHorizontalLayout.addWidget(self.shortMsg)
        self.sendMsgButton = QtWidgets.QPushButton(parent=self.centralwidget)
        self.sendMsgButton.setObjectName("sendMsgButton")
        self.msgHorizontalLayout.addWidget(self.sendMsgButton)
        self.centralGridLayout.addLayout(self.msgHorizontalLayout, 6, 0, 1, 1)
        FmTransfer.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(parent=FmTransfer)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 573, 22))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(parent=self.menubar)
        self.menuFile.setObjectName("menuFile")
        FmTransfer.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(parent=FmTransfer)
        self.statusbar.setObjectName("statusbar")
        FmTransfer.setStatusBar(self.statusbar)
        self.menubar.addAction(self.menuFile.menuAction())

        self.retranslateUi(FmTransfer)
        self.ggProtocolComboBox.setCurrentIndex(2)
        self.checkSignalButton.clicked.connect(FmTransfer.check_signal) # type: ignore
        self.dsrRadioButton.toggled['bool'].connect(FmTransfer.toggle_signal) # type: ignore
        self.sendMsgButton.clicked.connect(FmTransfer.send_text) # type: ignore
        self.receiveButton.clicked.connect(FmTransfer.receive_file) # type: ignore
        self.sendFileButton.clicked.connect(FmTransfer.send_file) # type: ignore
        self.chooseSendFileButton.clicked.connect(FmTransfer.choose_send_file) # type: ignore
        self.ggProtocolComboBox.currentIndexChanged['int'].connect(FmTransfer.set_gg_protocol) # type: ignore
        self.chooseRecvFileButton.clicked.connect(FmTransfer.choose_recv_file) # type: ignore
        self.recvMsgButton.clicked.connect(FmTransfer.receive_text) # type: ignore
        self.pttReleasedRadioButton.toggled['bool'].connect(FmTransfer.toggle_ptt) # type: ignore
        self.ggRadioButton.toggled['bool'].connect(FmTransfer.set_tool) # type: ignore
        self.quietProtocolComboBox.currentIndexChanged['int'].connect(FmTransfer.set_quiet_protocol) # type: ignore
        self.pushButton.clicked.connect(FmTransfer.recheck_serial_ports) # type: ignore
        self.signalLogic.clicked['bool'].connect(FmTransfer.set_signal_logic) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(FmTransfer)
        FmTransfer.setTabOrder(self.pttReleasedRadioButton, self.pttPressedRadioButton)
        FmTransfer.setTabOrder(self.pttPressedRadioButton, self.ggRadioButton)
        FmTransfer.setTabOrder(self.ggRadioButton, self.quietRadioButton)
        FmTransfer.setTabOrder(self.quietRadioButton, self.ggProtocolComboBox)
        FmTransfer.setTabOrder(self.ggProtocolComboBox, self.quietProtocolComboBox)
        FmTransfer.setTabOrder(self.quietProtocolComboBox, self.dsrRadioButton)
        FmTransfer.setTabOrder(self.dsrRadioButton, self.rtsRadioButton)
        FmTransfer.setTabOrder(self.rtsRadioButton, self.messages)
        FmTransfer.setTabOrder(self.messages, self.checkSignalButton)
        FmTransfer.setTabOrder(self.checkSignalButton, self.serialComboBox)
        FmTransfer.setTabOrder(self.serialComboBox, self.chooseSendFileButton)
        FmTransfer.setTabOrder(self.chooseSendFileButton, self.sendFileLineEdit)
        FmTransfer.setTabOrder(self.sendFileLineEdit, self.sendFileButton)
        FmTransfer.setTabOrder(self.sendFileButton, self.chooseRecvFileButton)
        FmTransfer.setTabOrder(self.chooseRecvFileButton, self.receiveFileLineEdit)
        FmTransfer.setTabOrder(self.receiveFileLineEdit, self.receiveButton)
        FmTransfer.setTabOrder(self.receiveButton, self.recvMsgButton)
        FmTransfer.setTabOrder(self.recvMsgButton, self.shortMsg)
        FmTransfer.setTabOrder(self.shortMsg, self.sendMsgButton)

    def retranslateUi(self, FmTransfer):
        _translate = QtCore.QCoreApplication.translate
        FmTransfer.setWindowTitle(_translate("FmTransfer", "FM-Transfer"))
        self.pttGroupBox.setTitle(_translate("FmTransfer", "PTT"))
        self.pttReleasedRadioButton.setText(_translate("FmTransfer", "Unpressed"))
        self.pttPressedRadioButton.setText(_translate("FmTransfer", "Pressed"))
        self.signalLogic.setText(_translate("FmTransfer", "Reverse logic"))
        self.toolGroupBox.setTitle(_translate("FmTransfer", "Tool"))
        self.ggRadioButton.setText(_translate("FmTransfer", "gg-transfer"))
        self.quietRadioButton.setText(_translate("FmTransfer", "quiet-lib"))
        self.protocolGroupBox.setTitle(_translate("FmTransfer", "Send protocol"))
        self.ggProtocolComboBox.setItemText(0, _translate("FmTransfer", "Normal"))
        self.ggProtocolComboBox.setItemText(1, _translate("FmTransfer", "Fast"))
        self.ggProtocolComboBox.setItemText(2, _translate("FmTransfer", "Fastest"))
        self.ggProtocolComboBox.setItemText(3, _translate("FmTransfer", "[U] Normal"))
        self.ggProtocolComboBox.setItemText(4, _translate("FmTransfer", "[U] Fast"))
        self.ggProtocolComboBox.setItemText(5, _translate("FmTransfer", "[U] Fastest"))
        self.ggProtocolComboBox.setItemText(6, _translate("FmTransfer", "[DT] Normal"))
        self.ggProtocolComboBox.setItemText(7, _translate("FmTransfer", "[DT] Fast"))
        self.ggProtocolComboBox.setItemText(8, _translate("FmTransfer", "[DT] Fastest"))
        self.quietProtocolComboBox.setItemText(0, _translate("FmTransfer", "audible"))
        self.quietProtocolComboBox.setItemText(1, _translate("FmTransfer", "audible-7k-channel-0"))
        self.quietProtocolComboBox.setItemText(2, _translate("FmTransfer", "audible-7k-channel-1"))
        self.quietProtocolComboBox.setItemText(3, _translate("FmTransfer", "cable-64k"))
        self.quietProtocolComboBox.setItemText(4, _translate("FmTransfer", "ultrasonic"))
        self.quietProtocolComboBox.setItemText(5, _translate("FmTransfer", "ultrasonic-3600"))
        self.quietProtocolComboBox.setItemText(6, _translate("FmTransfer", "ultrasonic-whisper"))
        self.signalGroupBox.setTitle(_translate("FmTransfer", "Signal"))
        self.dsrRadioButton.setText(_translate("FmTransfer", "DTR"))
        self.rtsRadioButton.setText(_translate("FmTransfer", "RTS"))
        self.checkSignalButton.setText(_translate("FmTransfer", "Check signal"))
        self.pushButton.setText(_translate("FmTransfer", "Recheck Serial Ports"))
        self.chooseSendFileButton.setText(_translate("FmTransfer", "Choose send file"))
        self.sendFileButton.setText(_translate("FmTransfer", "Send file"))
        self.chooseRecvFileButton.setText(_translate("FmTransfer", "Choose recv file"))
        self.receiveButton.setText(_translate("FmTransfer", "Receive file"))
        self.recvMsgButton.setText(_translate("FmTransfer", "Receive"))
        self.shortMsg.setPlaceholderText(_translate("FmTransfer", "Short message"))
        self.sendMsgButton.setText(_translate("FmTransfer", "Send"))
        self.menuFile.setTitle(_translate("FmTransfer", "File"))
from led import Led



import os
import re
import tempfile
from pathlib import Path
from typing import Any, Optional

import serial.serialutil
from PyQt6.QtCore import QSettings
from PyQt6.QtGui import QCloseEvent
from PyQt6.QtWidgets import QMainWindow, QFileDialog
from PyQt6.QtSerialPort import QSerialPortInfo
from PyQt6 import QtCore
from serial import Serial
from .FmWindow import Ui_FmTransfer


class FmTransfer(QMainWindow, Ui_FmTransfer):

    def __init__(self, **kwargs: Any) -> None:
        super(FmTransfer, self).__init__(**kwargs)
        self._tool = True
        self._quiet_protocol_list = ["audible", "audible-7k-channel-0", "audible-7k-channel-1",
                                     "cable-64k", "ultrasonic",
                                     "ultrasonic-3600", "ultrasonic-whisper"]
        self._quiet_protocol = 0
        self._gg_protocol = 2
        self._logic = True
        self.setupUi(FmTransfer=self)  # type: ignore[no-untyped-call]
        self._serialdevice: Optional[Serial] = None
        self._serial = ""
        self._serial_index = 0
        self._serial_closed = True
        self._com_ports = ["none"]
        self.recheck_serial_ports()
        self.pttPressedRadioButton.toggled['bool'].connect(self.led.setOn)
        self.pttGroupBox.setDisabled(True)
        self.signalGroupBox.setDisabled(True)
        self.checkSignalButton.setDisabled(True)
        self.led.setDisabled(True)
        self._ptt_unpressed: bool = True
        self._signal_dtr = True  # DSR, True = RTS
        self._send_filename: Optional[str] = None
        self._receive_filename: Optional[str] = None
        self._ggtransfer = True
        self._process_send = QtCore.QProcess(self)
        self._process_send.readyRead.connect(self._send_data_ready)
        self._process_send.started.connect(self._send_data_started)
        self._process_send.finished.connect(self._send_data_end)
        self._process_send_msg = QtCore.QProcess(self)
        self._process_send_msg.readyRead.connect(self._send_msg_data_ready)
        self._process_send_msg.started.connect(self._send_msg_data_started)
        self._process_send_msg.finished.connect(self._send_msg_data_end)
        self._process_receive = QtCore.QProcess(self)
        self._process_receive.readyRead.connect(self._receive_data_ready)
        self._process_receive.started.connect(self._receive_data_started)
        self._process_receive.finished.connect(self._receive_data_end)
        self._process_receive_msg = QtCore.QProcess(self)
        self._process_receive_msg.readyRead.connect(self._receive_msg_data_ready)
        self._process_receive_msg.started.connect(self._receive_msg_data_started)
        self._process_receive_msg.finished.connect(self._receive_msg_data_end)
        self._receive_in_progress = False
        self._send_in_progress = False
        self.progressBar.setMaximum(1)
        self.progressBar.setValue(0)
        self._pieces = 1
        self._current_piece = 0
        self._protocol = 0
        self._crc32 = ""
        self._tempfile: Optional[int] = None
        self._tempfilename: Optional[str] = None
        self._load_settings()

    def closeEvent(self, event: QCloseEvent) -> None:  # type: ignore[override]
        # settings = QSettings(r"r:\settings.ini", QSettings.Format.IniFormat)
        settings = QSettings(QSettings.Format.IniFormat, QSettings.Scope.UserScope, "fm-transfer",
                             "fm-transfer")
        settings.setValue("MainWindow/geometry", self.saveGeometry())
        settings.setValue("MainWindow/state", self.saveState())
        if self._receive_filename is not None:
            settings.setValue("MainWindow/receivefile", self._receive_filename)
        if self._send_filename is not None:
            settings.setValue("MainWindow/sendfile", self._send_filename)
        settings.setValue("MainWindow/tool", self._tool)
        settings.setValue("MainWindow/quiet_protocol", self._quiet_protocol)
        settings.setValue("MainWindow/gg_protocol", self._gg_protocol)
        settings.setValue("MainWindow/signal", self._signal_dtr)
        settings.setValue("MainWindow/serial", self._serial)
        settings.setValue("MainWindow/serial_index", self._serial_index)
        settings.setValue("MainWindow/signal_logic", not self._logic)
        super().closeEvent(event)

    def _load_settings(self) -> None:
        # settings = QSettings(r"r:\settings.ini", QSettings.Format.IniFormat)
        settings = QSettings(QSettings.Format.IniFormat, QSettings.Scope.UserScope, "fm-transfer",
                             "fm-transfer")
        self.restoreGeometry(settings.value("MainWindow/geometry", self.saveGeometry()))
        self.restoreState(settings.value("MainWindow/state", self.saveState()))
        send_filename = settings.value("MainWindow/sendfile", None)
        if send_filename is not None:
            self._send_filename = send_filename
            assert self._send_filename is not None
            short_fname = ('...' + self._send_filename[-20:]) if len(
                self._send_filename) > 20 else self._send_filename
            self.sendFileLineEdit.setText(short_fname)
        receive_filename = settings.value("MainWindow/receivefile", None)
        if receive_filename is not None:
            self._receive_filename = receive_filename
            assert self._receive_filename is not None
            short_fname = ('...' + self._receive_filename[-20:]) if len(
                self._receive_filename) > 20 else self._receive_filename
            self.receiveFileLineEdit.setText(short_fname)

        self._tool = settings.value("MainWindow/tool", self._tool, bool)
        self.quietRadioButton.setChecked(self._tool)
        if self._tool:
            self.set_tool(self._tool)

        gg_protocol = settings.value("MainWindow/gg_protocol", self._gg_protocol, int)
        self.ggProtocolComboBox.setCurrentIndex(gg_protocol)
        quiet_protocol = settings.value("MainWindow/quiet_protocol", self._quiet_protocol, int)
        self.quietProtocolComboBox.setCurrentIndex(quiet_protocol)

        self._logic = settings.value("MainWindow/signal_logic", self._logic, bool)
        self.signalLogic.setChecked(self._logic)
        self.set_signal_logic(self._logic)

        signal = settings.value("MainWindow/signal", self._signal_dtr, bool)
        self._signal_dtr = signal
        self.rtsRadioButton.setChecked(not self._signal_dtr)

        serial_name = settings.value("MainWindow/serial", "")
        serial_index = settings.value("MainWindow/serial_index", self._serial_index, int)
        if serial_index < len(self._com_ports):
            if self._com_ports[serial_index] == serial_name:
                self.serialComboBox.setCurrentIndex(serial_index)

    def _disable_interface_elements(self, mode: str) -> None:

        self.checkSignalButton.setDisabled(True)

        self.led.setDisabled(True)

        self.protocolGroupBox.setDisabled(True)
        self.pttGroupBox.setDisabled(True)
        self.signalGroupBox.setDisabled(True)
        self.toolGroupBox.setDisabled(True)

        self.serialComboBox.setDisabled(True)

        self.shortMsg.setDisabled(True)

        self.chooseSendFileButton.setDisabled(True)
        self.chooseRecvFileButton.setDisabled(True)

        if mode == "send_file":
            self.sendFileButton.setText("Stop sending")
            self.receiveButton.setDisabled(True)
            self.sendMsgButton.setDisabled(True)
            self.recvMsgButton.setDisabled(True)
        elif mode == "receive_file":
            self.sendFileButton.setDisabled(True)
            self.receiveButton.setText("Stop receiving")
            self.sendMsgButton.setDisabled(True)
            self.recvMsgButton.setDisabled(True)
        elif mode == "send_msg":
            self.sendFileButton.setDisabled(True)
            self.receiveButton.setDisabled(True)
            # self.sendMsgButton.setDisabled(True)
            self.sendMsgButton.setText("Sending...")
            self.recvMsgButton.setDisabled(True)
        elif mode == "receive_msg":
            self.sendFileButton.setDisabled(True)
            self.receiveButton.setDisabled(True)
            self.sendMsgButton.setDisabled(True)
            self.recvMsgButton.setText("Receiving...")
            # self.recvMsgButton.setDisabled(True)
        else:
            raise ValueError(f"Invalid mode: {mode}")

    def _enable_interface_elements(self) -> None:

        self.checkSignalButton.setDisabled(self._serial_closed)

        self.led.setDisabled(self._serial_closed)

        self.protocolGroupBox.setDisabled(False)
        self.pttGroupBox.setDisabled(self._serial_closed)
        self.signalGroupBox.setDisabled(self._serial_closed)
        self.toolGroupBox.setDisabled(False)

        self.serialComboBox.setDisabled(False)

        self.shortMsg.setDisabled(False)

        self.chooseSendFileButton.setDisabled(False)
        self.chooseRecvFileButton.setDisabled(False)

        _trans = QtCore.QCoreApplication.translate
        self.sendFileButton.setText(_trans("FmTransfer", "Send file"))
        self.receiveButton.setText(_trans("FmTransfer", "Receive file"))
        self.recvMsgButton.setText(_trans("FmTransfer", "Receive"))
        self.sendMsgButton.setText(_trans("FmTransfer", "Send"))

        self.sendFileButton.setDisabled(False)
        self.receiveButton.setDisabled(False)
        self.sendMsgButton.setDisabled(False)
        self.recvMsgButton.setDisabled(False)

    def _send_msg_data_ready(self) -> None:
        new = str(self._process_send_msg.readAll().data(), 'utf-8')
        # new = re.sub(r"\r", "\n", new)
        for line in new.splitlines():
            if line.startswith("Piece "):
                match = re.match(r"Piece (\d+)/(\d+)", line)
                if match is not None:
                    self._current_piece = int(match.group(1))
                    self._pieces = int(match.group(2))
                    self.progressBar.setMaximum(self._pieces)
                    self.progressBar.setValue(self._current_piece)
            elif line.startswith("Speed"):
                self.messages.insertPlainText(line + "\n")
                self.messages.ensureCursorVisible()

    def _send_msg_data_started(self) -> None:
        self._pieces = 1
        self._current_piece = 0
        self.progressBar.setMaximum(self._pieces)
        self.progressBar.setValue(self._current_piece)
        self._disable_interface_elements("send_msg")

    def _send_msg_data_end(self) -> None:
        if self._tempfilename is not None:
            os.unlink(self._tempfilename)
            self._tempfilename = None
        self._send_in_progress = False
        if not self._serial_closed:
            self.pttReleasedRadioButton.setChecked(True)
        self._enable_interface_elements()

    def _receive_msg_data_ready(self) -> None:
        new = str(self._process_send_msg.readAll().data(), 'utf-8')
        # new = re.sub(r"\r", "\n", new)
        for line in new.splitlines():
            self.messages.insertPlainText(line + "\n")
            self.messages.ensureCursorVisible()

    def _receive_msg_data_started(self) -> None:
        self._pieces = 1
        self._current_piece = 0
        self.progressBar.setMaximum(self._pieces)
        self.progressBar.setValue(self._current_piece)
        self._disable_interface_elements("receive_msg")

    def _receive_msg_data_end(self) -> None:
        if self._tempfilename is not None:
            if os.path.isfile(self._tempfilename):
                with open(self._tempfilename, "rb") as f:
                    self._current_piece = 1
                    self.progressBar.setValue(self._current_piece)
                    self.messages.insertPlainText(f.read().decode("utf-8") + "\n")
                    self.messages.ensureCursorVisible()
                os.unlink(self._tempfilename)
            self._tempfilename = None
        self._receive_in_progress = False
        self._enable_interface_elements()

    def _send_data_ready(self) -> None:
        new = str(self._process_send.readAll().data(), 'utf-8')
        # new = re.sub(r"\r", "\n", new)
        for line in new.splitlines():
            if line.startswith("Pieces: "):
                match = re.match(r"Pieces: (\d+)", line)
                if match is not None:
                    self._pieces = int(match.group(1))
                    self.progressBar.setMaximum(self._pieces)
                    self.messages.insertPlainText(line + "\n")
            elif line.startswith("Piece "):
                match = re.match(r"Piece (\d+)", line)
                if match is not None:
                    self._current_piece = int(match.group(1))
                    self.progressBar.setValue(self._current_piece)
            elif line.startswith("Speed") or line.startswith("Sending") or line.startswith("Time"):
                self.messages.insertPlainText(line + "\n")
            elif line.startswith("Size:"):
                match = re.match(r"Size: (\d+)", line)
                if match is not None:
                    self._pieces = int(match.group(1))
                    self.progressBar.setMaximum(self._pieces)
                    self.messages.insertPlainText(line + "\n")
            elif line.startswith("Sent:"):
                match = re.match(r"Sent: (\d+)", line)
                if match is not None:
                    self._current_piece = int(match.group(1))
                    self.progressBar.setValue(self._current_piece)
        self.messages.ensureCursorVisible()

    def _send_data_started(self) -> None:
        self._pieces = 1
        self._current_piece = 0
        self.progressBar.setMaximum(self._pieces)
        self.progressBar.setValue(self._current_piece)
        self._disable_interface_elements("send_file")

    def _send_data_end(self) -> None:
        self._send_in_progress = False
        self._enable_interface_elements()
        if not self._serial_closed:
            self.pttReleasedRadioButton.setChecked(True)

    def _receive_data_ready(self) -> None:
        new = str(self._process_receive.readAll().data(), 'utf-8')
        # new = re.sub(r"\r", "\n", new)
        for line in new.splitlines():
            if line.startswith("Got header"):
                match = re.match(r".*pieces: (\d+)", line)
                if match is not None:
                    self._pieces = int(match.group(1))
                    self.progressBar.setMaximum(self._pieces)
                    self.messages.insertPlainText(f"Got pieces number: {self._pieces}" + "\n")
                match = re.match(r".*CRC32: (.{8}).*", line)
                if match is not None:
                    self._crc32 = match.group(1)
                    self.messages.insertPlainText(f"Got CRC: {self._crc32}" + "\n")
                self.messages.insertPlainText(line + "\n")
            elif line.startswith("Piece "):
                match = re.match(r"Piece (\d+)", line)
                if match is not None:
                    self._current_piece = int(match.group(1))
                    self.progressBar.setValue(self._current_piece)
            elif line.startswith("Speed") or line.find("ERROR") != -1:
                self.messages.insertPlainText(line + "\n")
            elif line.startswith("Size:"):
                match = re.match(r"Size: (\d+)", line)
                if match is not None:
                    self._pieces = int(match.group(1))
                    self.progressBar.setMaximum(self._pieces)
                    self.messages.insertPlainText(line + "\n")
            elif line.startswith("Received:"):
                match = re.match(r"Received: (\d+)", line)
                if match is not None:
                    self._current_piece = int(match.group(1))
                    self.progressBar.setValue(self._current_piece)
            elif line.startswith("Time") or line.startswith("Speed") or line.startswith("CRC"):
                self.messages.insertPlainText(line + "\n")

        self.messages.ensureCursorVisible()

    def _receive_data_started(self) -> None:
        self._pieces = 1
        self._current_piece = 0
        self.progressBar.setMaximum(self._pieces)
        self.progressBar.setValue(self._current_piece)
        self._disable_interface_elements("receive_file")

    def _receive_data_end(self) -> None:
        self._enable_interface_elements()
        self._receive_in_progress = False
        # if self.pttGroupBox.isEnabled()
        # if not self._serial_closed:
        #     self.pttReleasedRadioButton.setChecked(True)

    # noinspection PyUnresolvedReferences
    def reinit_serial(self, index: int) -> None:
        self._serial_index = index
        if self._serialdevice and self._serialdevice.is_open:
            self._serialdevice.close()
            self._serialdevice = None
            self._serial_closed = True
        if self._serial_index == 0:
            self._serialdevice = None
            self.pttGroupBox.setDisabled(True)
            self.signalGroupBox.setDisabled(True)
            self.checkSignalButton.setDisabled(True)
            self._serial_closed = True
            self.led.setDisabled(True)
        else:
            self._serial = self._com_ports[self._serial_index]
            self._serialdevice = Serial(baudrate=9600)
            self._serialdevice.dtr = True
            self._serialdevice.rts = True
            self._serialdevice.port = self._serial
            try:
                self._serial_closed = not self._serialdevice.is_open
                if self._serial_closed:
                    self._serialdevice.open()
                    self._serial_closed = False
                    self.pttGroupBox.setDisabled(False)
                    self.signalGroupBox.setDisabled(False)
                    self.led.setDisabled(False)
                    self.checkSignalButton.setDisabled(False)
                    self._send_signal()
                    self.check_signal()
            except serial.serialutil.SerialException as e:
                self.messages.insertPlainText(str(e) + "\n")
                self.messages.ensureCursorVisible()
                self.serialComboBox.setCurrentIndex(0)
            except Exception as e:
                self.messages.insertPlainText(str(e) + "\n")
                self.messages.ensureCursorVisible()
                self.serialComboBox.setCurrentIndex(0)

    # noinspection PyUnresolvedReferences
    def _send_signal(self) -> None:
        if self._serialdevice and self._serialdevice.is_open:
            # print("ptt", self._ptt_unpressed)
            if self._ptt_unpressed:
                self._serialdevice.dtr = self._logic
                self._serialdevice.rts = self._logic
            else:
                # print("signal", self._signal_dtr)
                if self._signal_dtr:
                    self._serialdevice.dtr = not self._logic
                    self._serialdevice.rts = self._logic
                else:
                    self._serialdevice.dtr = self._logic
                    self._serialdevice.rts = not self._logic

    def check_signal(self) -> None:
        # noinspection PyUnresolvedReferences
        if self._serialdevice and self._serialdevice.is_open:
            if self._serialdevice.dtr:
                self.messages.insertPlainText("DTR signal DOWN\n")
            else:
                self.messages.insertPlainText("DTR signal UP\n")
            if self._serialdevice.rts:
                self.messages.insertPlainText("RTS signal DOWN\n")
            else:
                self.messages.insertPlainText("RTS signal UP\n")
        self.messages.ensureCursorVisible()

    def toggle_ptt(self, checked: bool) -> None:
        # noinspection PyUnresolvedReferences
        if not self._serial_closed:
            self._ptt_unpressed = checked
            self._send_signal()
            self.check_signal()

    def toggle_signal(self, signal: bool) -> None:
        # noinspection PyUnresolvedReferences
        if not self._serial_closed:
            self._signal_dtr = signal
            self._send_signal()
            self.check_signal()

    def recheck_serial_ports(self) -> None:

        if not self._serial_closed and isinstance(self._serialdevice, Serial):
            self.pttReleasedRadioButton.setChecked(True)
            self._serialdevice.close()
            self._serial_closed = True
            self._serial = ""
            self._serialdevice = None
        try:
            self.serialComboBox.currentIndexChanged.disconnect(self.reinit_serial)
        except TypeError:
            pass
        self.serialComboBox.clear()
        self._com_ports = ["none"]
        self.serialComboBox.addItem("Choose a serial port...")
        available_ports = QSerialPortInfo.availablePorts()
        for port in available_ports:
            self._com_ports.append(port.systemLocation())
            self.serialComboBox.addItem(port.systemLocation())
        self.serialComboBox.currentIndexChanged.connect(self.reinit_serial)

    def send_file(self) -> None:
        if not self._send_in_progress:
            if not self._send_filename or not Path(self._send_filename).is_file():
                self.choose_send_file()
            if self._send_filename is not None and Path(self._send_filename).is_file():
                if not self._serial_closed and self.pttGroupBox.isEnabled():
                    self.pttPressedRadioButton.setChecked(True)
                if self._tool:
                    ex = "gg-transfer"
                    argss = f"send -p {self._gg_protocol} -f -i".split(" ")
                    # noinspection PyTypeChecker
                    argss.append(self._send_filename)
                else:
                    ex = "quiet-transfer"
                    argss = f"send -p {self._quiet_protocol_list[self._quiet_protocol]} -f -i".split(" ")
                    # noinspection PyTypeChecker
                    argss.append(self._send_filename)
                self._process_send.setProcessChannelMode(
                    QtCore.QProcess.ProcessChannelMode.MergedChannels)
                self._process_send.start(ex, argss)
                self._send_in_progress = True
        else:
            self._process_send.kill()
            self._send_in_progress = False
            self._enable_interface_elements()
            if not self._serial_closed and self.pttGroupBox.isEnabled():
                self.pttReleasedRadioButton.setChecked(True)

    def receive_file(self) -> None:
        if not self._receive_in_progress:
            if not self._receive_filename:
                self.choose_recv_file()
            if self._receive_filename:
                if self._tool:
                    ex = "gg-transfer"
                    argss = "receive -f -w -o".split(" ")
                    # noinspection PyTypeChecker
                    argss.append(self._receive_filename)
                else:
                    ex = "quiet-transfer"
                    argss = (f"receive -f -p {self._quiet_protocol_list[self._quiet_protocol]}"
                             f" -w -o").split(" ")
                    # noinspection PyTypeChecker
                    argss.append(self._receive_filename)
                # if not self._serial_closed and self.pttGroupBox.isEnabled():
                #     self.pttPressedRadioButton.setChecked(True)
                self._process_receive.setProcessChannelMode(
                    QtCore.QProcess.ProcessChannelMode.MergedChannels)
                self._process_receive.start(ex, argss)
                self._receive_in_progress = True
        else:
            self._process_receive.kill()
            self._receive_in_progress = False
            self._enable_interface_elements()
            # if not self._serial_closed and self.pttGroupBox.isEnabled():
            #     self.pttReleasedRadioButton.setChecked(True)

    def send_text(self) -> None:
        if not self._send_in_progress:
            msg = self.shortMsg.text()
            if msg:
                self._tempfile, self._tempfilename = tempfile.mkstemp(text=False)
                os.write(self._tempfile, msg.encode("utf-8"))
                os.close(self._tempfile)
                if not self._serial_closed and self.pttGroupBox.isEnabled():
                    self.pttPressedRadioButton.setChecked(True)
                ex = "gg-transfer"
                argss = f"send -p {self._protocol} -i {self._tempfilename}".split(" ")
                self._process_send_msg.setProcessChannelMode(
                    QtCore.QProcess.ProcessChannelMode.MergedChannels)
                self._process_send_msg.start(ex, argss)
                self._send_in_progress = True
        else:
            self._process_send_msg.kill()
            self._send_in_progress = False
            if not self._serial_closed and self.pttGroupBox.isEnabled():
                self.pttReleasedRadioButton.setChecked(True)
            self._enable_interface_elements()

    def receive_text(self) -> None:
        if not self._receive_in_progress:
            self._tempfile, self._tempfilename = tempfile.mkstemp(text=False)
            os.close(self._tempfile)
            ex = "gg-transfer"
            argss = f"receive -n 1 -w -o {self._tempfilename}".split(" ")
            self._process_receive_msg.setProcessChannelMode(
                QtCore.QProcess.ProcessChannelMode.SeparateChannels)
            self._process_receive_msg.start(ex, argss)
            self._receive_in_progress = True
        else:
            self._process_receive_msg.kill()
            self._receive_in_progress = False
            self._enable_interface_elements()

    def set_gg_protocol(self, protocol: int) -> None:
        self._gg_protocol = protocol

    def set_quiet_protocol(self, protocol: int) -> None:
        self._quiet_protocol = protocol

    def set_tool(self, tool: bool) -> None:
        self._tool = tool
        self.ggProtocolComboBox.setDisabled(not self._tool)
        self.quietProtocolComboBox.setDisabled(self._tool)

    def choose_recv_file(self) -> None:
        file_dialog = QFileDialog(self)
        file_dialog.setFileMode(QFileDialog.FileMode.AnyFile)
        file_dialog.setAcceptMode(QFileDialog.AcceptMode.AcceptSave)
        if file_dialog.exec():
            selected_files = file_dialog.selectedFiles()
            self._receive_filename = selected_files[0]
            short_fname = ('...' + self._receive_filename[-20:]) if len(
                self._receive_filename) > 20 else self._receive_filename
            self.receiveFileLineEdit.setText(short_fname)

    def choose_send_file(self) -> None:
        file_dialog = QFileDialog(self)
        file_dialog.setFileMode(QFileDialog.FileMode.ExistingFile)
        if file_dialog.exec():
            selected_files = file_dialog.selectedFiles()
            self._send_filename = selected_files[0]
            short_fname = ('...' + self._send_filename[-20:]) if len(
                self._send_filename) > 20 else self._send_filename
            self.sendFileLineEdit.setText(short_fname)

    def set_signal_logic(self, logic: bool) -> None:
        self._logic = not logic
        self._send_signal()
        self.check_signal()



from typing import Optional

from PyQt6.QtWidgets import QGraphicsDropShadowEffect, QWidget
from PyQt6.QtGui import QColor, QPainter, QPen, QRadialGradient, QMouseEvent, QPaintEvent
from PyQt6.QtCore import Qt, QPointF, pyqtSlot, QObject

# noinspection PyPep8Naming
class Led(QWidget):
    def __init__(self, parent: Optional[QWidget] = None):
        super().__init__(parent=parent)
        self.setFixedSize(24, 24)
        self.on = True
        self.light_color = QColor.fromRgbF(255, 0, 0)
        self.glow_effect = QGraphicsDropShadowEffect()
        self.glow_effect.setBlurRadius(48)
        self.glow_effect.setColor(self.light_color.lighter(150))
        self.glow_effect.setOffset(0, 0)
        self.glow_effect.setEnabled(False)
        self.setGraphicsEffect(self.glow_effect)
        self._parent: Optional[QObject] = None
        first_parent: Optional[QObject] = self.parent()
        if isinstance(first_parent, QWidget):
            self._parent = first_parent.parent()


    def isOn(self) -> bool:
        return self.on

    @pyqtSlot(bool)
    def setOn(self, on: bool) -> None:
        self.on = on
        self.glow_effect.setEnabled(self.on)
        self.update()

    def getLightColor(self) -> QColor:
        return self.light_color

    def mouseReleaseEvent(self, e: Optional[QMouseEvent]) -> None:
        if isinstance(self._parent, QWidget):
            if self._parent.pttPressedRadioButton.isChecked():  # type: ignore[attr-defined]
                self._parent.pttReleasedRadioButton.setChecked(True)  # type: ignore[attr-defined]
            else:
                self._parent.pttPressedRadioButton.setChecked(True)  # type: ignore[attr-defined]
        if e is not None:
            e.accept()

    @pyqtSlot(QColor)
    def setLightColor(self, color: QColor) -> None:
        self.light_color = color
        self.glow_effect.setColor(self.light_color.lighter(150))
        self.update()

    def paintEvent(self, event: Optional[QPaintEvent]) -> None:

        painter = QPainter(self)
        painter.setRenderHints(QPainter.RenderHint.Antialiasing)

        led_color = self.light_color if self.on else QColor(Qt.GlobalColor.darkGray)

        # defining a radial gradient to render the LED color and simulate a clear ambient reflection on its surface
        radius = self.width() / 2.0
        radial_gradient = QRadialGradient(QPointF(radius, radius), radius - 1, QPointF(radius, radius / 2.0))
        radial_gradient.setColorAt(0, led_color.lighter(300))
        radial_gradient.setColorAt(0.5, led_color)
        radial_gradient.setColorAt(1, led_color.darker(200))

        # set Pen for the outline and Brush for the filling
        painter.setPen(QPen(Qt.GlobalColor.gray, 2))
        painter.setBrush(radial_gradient)

        # draws the gray outline with a single directive and fills the circular area of the LED with the gradient
        painter.drawEllipse(QPointF(radius, radius), radius - 1, radius - 1)
